import asyncio
from functools import partial
from typing import Union
import httpx
from authlib.integrations.httpx_client import AsyncOAuth2Client


class MaxRetriesReached(Exception):
    pass


class IraAuth:
    token_endpoint = ""

    def __init__(self, client_id: str, client_secret: str, cookie_name: str = "token", token_endpoint: Union[str, None] = None):
        if token_endpoint != None:
            self.token_endpoint = token_endpoint
        self.oauth2_client = AsyncOAuth2Client(client_id, client_secret, scope="api", token_endpoint_auth_method="client_secret_post")
        self.http_client = httpx.AsyncClient(headers={"Accept": "application/json"})
        self.cookies = httpx.Cookies()
        self.cookie_name = cookie_name

    async def get_token(self) -> str:
        token = await self.oauth2_client.fetch_token(self.token_endpoint, grant_type="client_credentials")
        return token["access_token"]

    async def http(self, base_path: str, method: str, url:str, data: any = None, retry_after=5, max_retries=-1, _acc=0):
        try:
            response = await self.http_client.request(method, base_path + url, data=data, cookies=self.cookies)
            if response.status_code == httpx.codes.UNAUTHORIZED:
                if _acc > 0:
                    await asyncio.sleep(retry_after)
                
                if max_retries != -1 and _acc == max_retries:
                    raise MaxRetriesReached
                    
                token =  await self.get_token()
                self.cookies.set(self.cookie_name, token)
                _acc += 1
                response = await self.http(base_path, method, url, data, retry_after, max_retries, _acc)
                return response
            else:
                return response
        except Exception as e:
            print(e)
        return None

    def get_http_client(self, base_path: str=""):
        return partial(self.http, base_path)

    async def cleanup(self):
        await self.oauth2_client.aclose()
        await self.http_client.aclose()


import setuptools


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


setuptools.setup(
    name="ira_auth_m2m_client",
    version="0.1.0",
    author="Daryl Correa (Epicode)",
    author_email="daryl.correa@epicode.in",
    description="IraAuthM2M authenticates an application by IraAuth so that it can access APIs protected by IraAuth",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/epicira/ira-auth-m2m-client",
    project_urls = {
        "Bug Tracker": "https://github.com/epicira/ira-auth-m2m-client/issues"
    },
    license="MIT",
    packages=setuptools.find_packages("src"),
    package_dir={"": "src"},
    install_requires=[
        "httpx >= 0.23.3",
        "Authlib >= 1.2.0"
    ]
)
